@@ -449,10 +449,10 @@ p12U_ReadPKCS12File(SECItem *uniPwp, char *in_file, PK11SlotInfo *slot,
 
     /* revert the option setting */
     if (forceUnicode != pk12uForceUnicode) {
-        if (SECSuccess != NSS_OptionSet(__NSS_PKCS12_DECODE_FORCE_UNICODE, pk12uForceUnicode)) {
+        rv = NSS_OptionSet(__NSS_PKCS12_DECODE_FORCE_UNICODE, pk12uForceUnicode);
+        if (rv != SECSuccess) {
             SECU_PrintError(progName, "PKCS12 decoding failed to set option");
             pk12uErrno = PK12UERR_DECODEVERIFY;
-            rv = SECFailure;
         }
     }
     /* rv has been set at this point */
