@@ -17,6 +17,7 @@
 #include "secerr.h"
 #include "keyi.h"
 #include "nss.h"
+#include "prenv.h"
 
 /*
 ** Recover the DigestInfo from an RSA PKCS#1 signature.
@@ -139,19 +140,23 @@ struct VFYContextStr {
         unsigned char ecdsasig[2 * MAX_ECKEY_LEN];
         /* the full RSA signature, only used in RSA-PSS */
         unsigned char rsasig[(RSA_MAX_MODULUS_BITS + 7) / 8];
+        unsigned char gensig[MAX_SIGNATURE_LEN];
     } u;
+    unsigned int signatureLen;
     unsigned int pkcs1RSADigestInfoLen;
     /* the encoded DigestInfo from a RSA PKCS#1 signature */
     unsigned char *pkcs1RSADigestInfo;
     void *wincx;
     void *hashcx;
     const SECHashObject *hashobj;
-    SECOidTag encAlg;    /* enc alg */
+    PK11Context *vfycx;
+    SECOidTag encAlg; /* enc alg */
+    CK_MECHANISM_TYPE mech;
     PRBool hasSignature; /* true if the signature was provided in the
                           * VFY_CreateContext call.  If false, the
                           * signature must be provided with a
                           * VFY_EndWithSignature call. */
-    SECItem *params;
+    SECItem mechparams;
 };
 
 static SECStatus
@@ -189,6 +194,7 @@ checkedSignatureLen(const SECKEYPublicKey *pubk)
             PORT_SetError(SEC_ERROR_UNSUPPORTED_KEYALG);
             return 0;
     }
+    PORT_Assert(maxSigLen <= MAX_SIGNATURE_LEN);
     if (sigLen > maxSigLen) {
         PORT_SetError(SEC_ERROR_INVALID_KEY);
         return 0;
@@ -296,30 +302,146 @@ sec_GetEncAlgFromSigAlg(SECOidTag sigAlg)
     }
     return SEC_OID_UNKNOWN;
 }
+static CK_MECHANISM_TYPE
+sec_RSAPKCS1GetCombinedMech(SECOidTag hashalg)
+{
+    switch (hashalg) {
+        case SEC_OID_MD5:
+            return CKM_MD5_RSA_PKCS;
+        case SEC_OID_MD2:
+            return CKM_MD2_RSA_PKCS;
+        case SEC_OID_SHA1:
+            return CKM_SHA1_RSA_PKCS;
+        case SEC_OID_SHA224:
+            return CKM_SHA224_RSA_PKCS;
+        case SEC_OID_SHA256:
+            return CKM_SHA256_RSA_PKCS;
+        case SEC_OID_SHA384:
+            return CKM_SHA384_RSA_PKCS;
+        case SEC_OID_SHA512:
+            return CKM_SHA512_RSA_PKCS;
+        default:
+            break;
+    }
+    return CKM_INVALID_MECHANISM;
+}
+
+static CK_MECHANISM_TYPE
+sec_RSAPSSGetCombinedMech(SECOidTag hashalg)
+{
+    switch (hashalg) {
+        case SEC_OID_SHA1:
+            return CKM_SHA1_RSA_PKCS_PSS;
+        case SEC_OID_SHA224:
+            return CKM_SHA224_RSA_PKCS_PSS;
+        case SEC_OID_SHA256:
+            return CKM_SHA256_RSA_PKCS_PSS;
+        case SEC_OID_SHA384:
+            return CKM_SHA384_RSA_PKCS_PSS;
+        case SEC_OID_SHA512:
+            return CKM_SHA512_RSA_PKCS_PSS;
+        default:
+            break;
+    }
+    return CKM_INVALID_MECHANISM;
+}
+
+static CK_MECHANISM_TYPE
+sec_DSAGetCombinedMech(SECOidTag hashalg)
+{
+    switch (hashalg) {
+        case SEC_OID_SHA1:
+            return CKM_DSA_SHA1;
+        case SEC_OID_SHA224:
+            return CKM_DSA_SHA224;
+        case SEC_OID_SHA256:
+            return CKM_DSA_SHA256;
+        case SEC_OID_SHA384:
+            return CKM_DSA_SHA384;
+        case SEC_OID_SHA512:
+            return CKM_DSA_SHA512;
+        default:
+            break;
+    }
+    return CKM_INVALID_MECHANISM;
+}
+static CK_MECHANISM_TYPE
+sec_ECDSAGetCombinedMech(SECOidTag hashalg)
+{
+    switch (hashalg) {
+        case SEC_OID_SHA1:
+            return CKM_ECDSA_SHA1;
+        case SEC_OID_SHA224:
+            return CKM_ECDSA_SHA224;
+        case SEC_OID_SHA256:
+            return CKM_ECDSA_SHA256;
+        case SEC_OID_SHA384:
+            return CKM_ECDSA_SHA384;
+        case SEC_OID_SHA512:
+            return CKM_ECDSA_SHA512;
+        default:
+            break;
+    }
+    return CKM_INVALID_MECHANISM;
+}
+
+static CK_MECHANISM_TYPE
+sec_GetCombinedMech(SECOidTag encalg, SECOidTag hashalg)
+{
+    switch (encalg) {
+        case SEC_OID_PKCS1_RSA_ENCRYPTION:
+            return sec_RSAPKCS1GetCombinedMech(hashalg);
+        case SEC_OID_PKCS1_RSA_PSS_SIGNATURE:
+            return sec_RSAPSSGetCombinedMech(hashalg);
+        case SEC_OID_ANSIX962_EC_PUBLIC_KEY:
+            return sec_ECDSAGetCombinedMech(hashalg);
+        case SEC_OID_ANSIX9_DSA_SIGNATURE:
+            return sec_DSAGetCombinedMech(hashalg);
+        default:
+            break;
+    }
+    return CKM_INVALID_MECHANISM;
+}
 
 /*
  * Pulls the hash algorithm, signing algorithm, and key type out of a
  * composite algorithm.
  *
+ * key: pointer to the public key. Should be NULL if called for a sign operation.
  * sigAlg: the composite algorithm to dissect.
  * hashalg: address of a SECOidTag which will be set with the hash algorithm.
  * encalg: address of a SECOidTag which will be set with the signing alg.
+ * mechp: address of a PCKS #11 Mechanism which will be set to the
+ *  combined hash/encrypt mechanism. If set to CKM_INVALID_MECHANISM, the code
+ *  will fall back to external hashing.
+ * mechparams: address of a SECItem will set to the parameters for the combined
+ *  hash/encrypt mechanism.
  *
  * Returns: SECSuccess if the algorithm was acceptable, SECFailure if the
  *  algorithm was not found or was not a signing algorithm.
  */
 SECStatus
 sec_DecodeSigAlg(const SECKEYPublicKey *key, SECOidTag sigAlg,
-                 const SECItem *param, SECOidTag *encalgp, SECOidTag *hashalg)
+                 const SECItem *param, SECOidTag *encalgp, SECOidTag *hashalg,
+                 CK_MECHANISM_TYPE *mechp, SECItem *mechparamsp)
 {
     unsigned int len;
     PLArenaPool *arena;
     SECStatus rv;
     SECItem oid;
     SECOidTag encalg;
+    char *evp;
 
     PR_ASSERT(hashalg != NULL);
     PR_ASSERT(encalgp != NULL);
+    PR_ASSERT(mechp != NULL);
+    /* Get the expected combined mechanism from the signature OID
+     * We'll override it in the table below if necessary */
+    *mechp = PK11_AlgtagToMechanism(sigAlg);
+    if (mechparamsp) {
+        mechparamsp->data = NULL;
+        mechparamsp->len = 0;
+    }
 
     switch (sigAlg) {
         /* We probably shouldn't be generating MD2 signatures either */
@@ -335,24 +457,57 @@ sec_DecodeSigAlg(const SECKEYPublicKey *key, SECOidTag sigAlg,
             *hashalg = SEC_OID_SHA1;
             break;
         case SEC_OID_PKCS1_RSA_ENCRYPTION:
+            /* SEC_OID_PKCS1_RSA_ENCRYPTION returns the generic
+             * CKM_RSA_PKCS mechanism, which isn't a combined mechanism.
+             * We don't have a hash, so we need to fall back to the old
+             * code which gets the hashalg by decoding the signature */
+            *mechp = CKM_INVALID_MECHANISM;
             *hashalg = SEC_OID_UNKNOWN; /* get it from the RSA signature */
             break;
         case SEC_OID_PKCS1_RSA_PSS_SIGNATURE:
+            /* SEC_OID_PKCS1_RSA_PSS_SIGNATURE returns the generic
+             * CKM_RSA_PSS_PKCS mechanism, which isn't a combined mechanism.
+             * If successful, we'll select the mechanism below, set it to
+             * invalid here incase we aren't successful */
+            *mechp = CKM_INVALID_MECHANISM;
+            CK_RSA_PKCS_PSS_PARAMS *rsapssmechparams = NULL;
+            CK_RSA_PKCS_PSS_PARAMS space;
+
+            /* if we don't have a mechanism parameter to put the data in
+             * we don't need to return it, just use a stack buffer */
+            if (mechparamsp == NULL) {
+                rsapssmechparams = &space;
+            } else {
+                rsapssmechparams = PORT_ZNew(CK_RSA_PKCS_PSS_PARAMS);
+            }
+            if (rsapssmechparams == NULL) {
+                return SECFailure;
+            }
             if (param && param->data) {
                 PORTCheapArenaPool tmpArena;
 
                 PORT_InitCheapArena(&tmpArena, DER_DEFAULT_CHUNKSIZE);
-                rv = sec_DecodeRSAPSSParams(&tmpArena.arena, param,
-                                            hashalg, NULL, NULL);
+                rv = sec_DecodeRSAPSSParamsToMechanism(&tmpArena.arena, param,
+                                                       rsapssmechparams, hashalg);
                 PORT_DestroyCheapArena(&tmpArena);
 
                 /* only accept hash algorithms */
                 if (rv != SECSuccess || HASH_GetHashTypeByOidTag(*hashalg) == HASH_AlgNULL) {
                     /* error set by sec_DecodeRSAPSSParams or HASH_GetHashTypeByOidTag */
+                    if (mechparamsp)
+                        PORT_Free(rsapssmechparams);
                     return SECFailure;
                 }
             } else {
                 *hashalg = SEC_OID_SHA1; /* default, SHA-1 */
+                rsapssmechparams->hashAlg = CKM_SHA_1;
+                rsapssmechparams->mgf = CKG_MGF1_SHA1;
+                rsapssmechparams->sLen = SHA1_LENGTH;
+            }
+            *mechp = sec_RSAPSSGetCombinedMech(*hashalg);
+            if (mechparamsp) {
+                mechparamsp->data = (unsigned char *)rsapssmechparams;
+                mechparamsp->len = sizeof(*rsapssmechparams);
             }
             break;
 
@@ -385,6 +540,7 @@ sec_DecodeSigAlg(const SECKEYPublicKey *key, SECOidTag sigAlg,
         case SEC_OID_MISSI_KEA_DSS:
         case SEC_OID_MISSI_KEA_DSS_OLD:
         case SEC_OID_MISSI_DSS_OLD:
+            *mechp = CKM_DSA_SHA1;
             *hashalg = SEC_OID_SHA1;
             break;
         case SEC_OID_ANSIX962_ECDSA_SIGNATURE_RECOMMENDED_DIGEST:
@@ -406,6 +562,7 @@ sec_DecodeSigAlg(const SECKEYPublicKey *key, SECOidTag sigAlg,
                 /* use the largest in this case */
                 *hashalg = SEC_OID_SHA512;
             }
+            *mechp = sec_ECDSAGetCombinedMech(*hashalg);
             break;
         case SEC_OID_ANSIX962_ECDSA_SIGNATURE_SPECIFIED_DIGEST:
             if (param == NULL) {
@@ -429,23 +586,110 @@ sec_DecodeSigAlg(const SECKEYPublicKey *key, SECOidTag sigAlg,
                 /* error set by HASH_GetHashTypeByOidTag */
                 return SECFailure;
             }
+            *mechp = sec_ECDSAGetCombinedMech(*hashalg);
             break;
         /* we don't implement MD4 hashes */
         case SEC_OID_PKCS1_MD4_WITH_RSA_ENCRYPTION:
         default:
+            *mechp = CKM_INVALID_MECHANISM;
             PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
             return SECFailure;
     }
 
     encalg = sec_GetEncAlgFromSigAlg(sigAlg);
     if (encalg == SEC_OID_UNKNOWN) {
+        *mechp = CKM_INVALID_MECHANISM;
+        SECITEM_FreeItem(mechparamsp, PR_FALSE);
         return SECFailure;
     }
     *encalgp = encalg;
+    /* for testing, we want to be able to turn off combo signatures to
+     * 1) make sure the fallback code is working correctly so we know
+     * we can handle cases where the fallback doesn't work.
+     * 2) make sure that the combo code is compatible with the non-combo
+     * versions.
+     * We know if we are signing or verifying based on the value of 'key'.
+     * Since key is a public key, then it's set to NULL for signing */
+    evp = PR_GetEnvSecure("NSS_COMBO_SIGNATURES");
+    if (evp) {
+        if (PORT_Strcasecmp(evp, "none") == 0) {
+            *mechp = CKM_INVALID_MECHANISM;
+        } else if (key && (PORT_Strcasecmp(evp, "signonly") == 0)) {
+            *mechp = CKM_INVALID_MECHANISM;
+        } else if (!key && (PORT_Strcasecmp(evp, "vfyonly") == 0)) {
+            *mechp = CKM_INVALID_MECHANISM;
+        }
+        /* anything else we take as use combo, which is the default */
+    }
 
     return SECSuccess;
 }
 
+SECStatus
+vfy_ImportPublicKey(VFYContext *cx)
+{
+    PK11SlotInfo *slot;
+    CK_OBJECT_HANDLE objID;
+
+    if (cx->key->pkcs11Slot &&
+        PK11_DoesMechanismFlag(cx->key->pkcs11Slot,
+                               cx->mech, CKF_VERIFY)) {
+        return SECSuccess;
+    }
+    slot = PK11_GetBestSlotWithAttributes(cx->mech, CKF_VERIFY, 0, cx->wincx);
+    if (slot == NULL) {
+        return SECFailure; /* can't find a slot, fall back to
+                           * normal processing */
+    }
+    objID = PK11_ImportPublicKey(slot, cx->key, PR_FALSE);
+    PK11_FreeSlot(slot);
+    return objID == CK_INVALID_HANDLE ? SECFailure : SECSuccess;
+}
+
+/* Sometimes there are differences between how DER encodes a
+ * signature and how it's encoded in PKCS #11. This function converts the
+ * DER form to the PKCS #11 form. it also verify signature length based
+ * on the key, and verifies that length will fit in our buffer. */
+static SECStatus
+vfy_SetPKCS11SigFromX509Sig(VFYContext *cx, const SECItem *sig)
+{
+    unsigned int sigLen;
+
+    /* skip the legacy RSA PKCS #11 case, it's always handled separately */
+    if ((cx->key->keyType == rsaKey) && (cx->mech == CKM_INVALID_MECHANISM) &&
+        (cx->encAlg != SEC_OID_PKCS1_RSA_PSS_SIGNATURE)) {
+        return SECSuccess;
+    }
+
+    sigLen = checkedSignatureLen(cx->key);
+    /* Check signature length is within limits */
+    if (sigLen == 0) {
+        /* error set by checkedSignatureLen */
+        return SECFailure;
+    }
+    if (sigLen > sizeof(cx->u)) {
+        PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
+        return SECFailure;
+    }
+    /* save the authenticated length */
+    cx->signatureLen = sigLen;
+    switch (cx->encAlg) {
+        case SEC_OID_ANSIX9_DSA_SIGNATURE:
+        case SEC_OID_ANSIX962_EC_PUBLIC_KEY:
+            /* decodeECorDSASignature will check sigLen == sig->len after padding */
+            return decodeECorDSASignature(cx->encAlg, sig, cx->u.buffer, sigLen);
+        default:
+            break;
+    }
+    /* all other cases, no transform needed, just copy the signature */
+    if (sig->len != sigLen) {
+        PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
+        return SECFailure;
+    }
+    PORT_Memcpy(cx->u.buffer, sig->data, sigLen);
+    return SECSuccess;
+}
+
 /*
  * we can verify signatures that come from 2 different sources:
  *  one in with the signature contains a signature oid, and the other
@@ -453,6 +697,17 @@ sec_DecodeSigAlg(const SECKEYPublicKey *key, SECOidTag sigAlg,
  *  and a hash oid. The latter is the more basic, so that's what
  *  our base vfyCreate function takes.
  *
+ * Modern signature algorithms builds the hashing into the algorithm, and
+ *  some tokens (like smart cards), purposefully only export hash & sign
+ *  combo mechanisms (which gives stronger guarrentees on key type usage
+ *  for RSA operations). If mech is set to a PKCS #11 mechanism, we assume
+ *  we can do the combined operations, otherwise we fall back to manually
+ *  hashing then signing.
+ *
+ * This function adopts the mechparamsp parameter, so if we fail before
+ *  setting up the context, we need to free any space associated with it
+ *  before we return.
+ *
  * There is one noteworthy corner case, if we are using an RSA key, and the
  * signature block is provided, then the hashAlg can be specified as
  * SEC_OID_UNKNOWN. In this case, verify will use the hash oid supplied
@@ -460,11 +715,12 @@ sec_DecodeSigAlg(const SECKEYPublicKey *key, SECOidTag sigAlg,
  */
 static VFYContext *
 vfy_CreateContext(const SECKEYPublicKey *key, const SECItem *sig,
-                  SECOidTag encAlg, SECOidTag hashAlg, SECOidTag *hash, void *wincx)
+                  SECOidTag encAlg, SECOidTag hashAlg, CK_MECHANISM_TYPE mech,
+                  SECItem *mechparamsp, SECOidTag *hash, PRBool prehash,
+                  void *wincx)
 {
     VFYContext *cx;
     SECStatus rv;
-    unsigned int sigLen;
     KeyType type;
     PRUint32 policyFlags;
     PRInt32 optFlags;
@@ -474,15 +730,17 @@ vfy_CreateContext(const SECKEYPublicKey *key, const SECItem *sig,
     type = seckey_GetKeyType(encAlg);
     if ((key->keyType != type) &&
         ((key->keyType != rsaKey) || (type != rsaPssKey))) {
+        SECITEM_FreeItem(mechparamsp, PR_FALSE);
         PORT_SetError(SEC_ERROR_PKCS7_KEYALG_MISMATCH);
         return NULL;
     }
     if (NSS_OptionGet(NSS_KEY_SIZE_POLICY_FLAGS, &optFlags) != SECFailure) {
         if (optFlags & NSS_KEY_SIZE_POLICY_VERIFY_FLAG) {
-            rv = seckey_EnforceKeySize(key->keyType,
+            rv = SECKEY_EnforceKeySize(key->keyType,
                                        SECKEY_PublicKeyStrengthInBits(key),
                                        SEC_ERROR_SIGNATURE_ALGORITHM_DISABLED);
             if (rv != SECSuccess) {
+                SECITEM_FreeItem(mechparamsp, PR_FALSE);
                 return NULL;
             }
         }
@@ -490,12 +748,16 @@ vfy_CreateContext(const SECKEYPublicKey *key, const SECItem *sig,
     /* check the policy on the encryption algorithm */
     if ((NSS_GetAlgorithmPolicy(encAlg, &policyFlags) == SECFailure) ||
         !(policyFlags & NSS_USE_ALG_IN_ANY_SIGNATURE)) {
+        SECITEM_FreeItem(mechparamsp, PR_FALSE);
         PORT_SetError(SEC_ERROR_SIGNATURE_ALGORITHM_DISABLED);
         return NULL;
     }
 
     cx = (VFYContext *)PORT_ZAlloc(sizeof(VFYContext));
     if (cx == NULL) {
+        /* after this point mechparamsp is 'owned' by the context and will be
+         * freed by Destroy context for any other failures here */
+        SECITEM_FreeItem(mechparamsp, PR_FALSE);
         goto loser;
     }
 
@@ -503,50 +765,46 @@ vfy_CreateContext(const SECKEYPublicKey *key, const SECItem *sig,
     cx->hasSignature = (sig != NULL);
     cx->encAlg = encAlg;
     cx->hashAlg = hashAlg;
+    cx->mech = mech;
+    if (mechparamsp) {
+        cx->mechparams = *mechparamsp;
+    } else {
+        /* probably needs to have a call to set the default
+         * paramseters based on hashAlg and encAlg */
+        cx->mechparams.data = NULL;
+        cx->mechparams.len = 0;
+    }
     cx->key = SECKEY_CopyPublicKey(key);
     cx->pkcs1RSADigestInfo = NULL;
     rv = SECSuccess;
+    if (mech != CKM_INVALID_MECHANISM) {
+        rv = vfy_ImportPublicKey(cx);
+        /* if we can't import the key, then we probably can't
+        * support the requested combined mechanism, fallback
+        * to the non-combined method */
+        if (rv != SECSuccess) {
+            cx->mech = mech = CKM_INVALID_MECHANISM;
+        }
+    }
     if (sig) {
         rv = SECFailure;
-        if (type == rsaKey) {
+        /* sigh, if we are prehashing, we still need to do verifyRecover
+         * recover for RSA PKCS #1 */
+        if ((mech == CKM_INVALID_MECHANISM || prehash) && (type == rsaKey)) {
+            /* in traditional rsa PKCS #1, we use verify recover to get
+             * the encoded RSADigestInfo. In all other cases we just
+             * stash the signature encoded in PKCS#11 in our context */
             rv = recoverPKCS1DigestInfo(hashAlg, &cx->hashAlg,
                                         &cx->pkcs1RSADigestInfo,
                                         &cx->pkcs1RSADigestInfoLen,
                                         cx->key,
                                         sig, wincx);
         } else {
-            sigLen = checkedSignatureLen(key);
-            /* Check signature length is within limits */
-            if (sigLen == 0) {
-                /* error set by checkedSignatureLen */
-                rv = SECFailure;
-                goto loser;
-            }
-            if (sigLen > sizeof(cx->u)) {
-                PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
-                rv = SECFailure;
-                goto loser;
-            }
-            switch (type) {
-                case rsaPssKey:
-                    if (sig->len != sigLen) {
-                        PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
-                        rv = SECFailure;
-                        goto loser;
-                    }
-                    PORT_Memcpy(cx->u.buffer, sig->data, sigLen);
-                    rv = SECSuccess;
-                    break;
-                case ecKey:
-                case dsaKey:
-                    /* decodeECorDSASignature will check sigLen == sig->len after padding */
-                    rv = decodeECorDSASignature(encAlg, sig, cx->u.buffer, sigLen);
-                    break;
-                default:
-                    /* Unreachable */
-                    rv = SECFailure;
-                    goto loser;
-            }
+            /* at this point hashAlg should be known. Only the RSA case
+             * enters here with hashAlg unknown, and it's found out
+             * above */
+            PORT_Assert(hashAlg != SEC_OID_UNKNOWN);
+            rv = vfy_SetPKCS11SigFromX509Sig(cx, sig);
         }
         if (rv != SECSuccess) {
             goto loser;
@@ -584,11 +842,15 @@ VFY_CreateContext(SECKEYPublicKey *key, SECItem *sig, SECOidTag sigAlg,
                   void *wincx)
 {
     SECOidTag encAlg, hashAlg;
-    SECStatus rv = sec_DecodeSigAlg(key, sigAlg, NULL, &encAlg, &hashAlg);
+    CK_MECHANISM_TYPE mech;
+    SECItem mechparams;
+    SECStatus rv = sec_DecodeSigAlg(key, sigAlg, NULL, &encAlg, &hashAlg,
+                                    &mech, &mechparams);
     if (rv != SECSuccess) {
         return NULL;
     }
-    return vfy_CreateContext(key, sig, encAlg, hashAlg, NULL, wincx);
+    return vfy_CreateContext(key, sig, encAlg, hashAlg, mech,
+                             &mechparams, NULL, PR_FALSE, wincx);
 }
 
 VFYContext *
@@ -596,28 +858,28 @@ VFY_CreateContextDirect(const SECKEYPublicKey *key, const SECItem *sig,
                         SECOidTag encAlg, SECOidTag hashAlg,
                         SECOidTag *hash, void *wincx)
 {
-    return vfy_CreateContext(key, sig, encAlg, hashAlg, hash, wincx);
+    CK_MECHANISM_TYPE mech = sec_GetCombinedMech(encAlg, hashAlg);
+    return vfy_CreateContext(key, sig, encAlg, hashAlg, mech, NULL,
+                             hash, PR_FALSE, wincx);
 }
 
 VFYContext *
 VFY_CreateContextWithAlgorithmID(const SECKEYPublicKey *key, const SECItem *sig,
                                  const SECAlgorithmID *sigAlgorithm, SECOidTag *hash, void *wincx)
 {
-    VFYContext *cx;
     SECOidTag encAlg, hashAlg;
+    CK_MECHANISM_TYPE mech;
+    SECItem mechparams;
     SECStatus rv = sec_DecodeSigAlg(key,
                                     SECOID_GetAlgorithmTag((SECAlgorithmID *)sigAlgorithm),
-                                    &sigAlgorithm->parameters, &encAlg, &hashAlg);
+                                    &sigAlgorithm->parameters, &encAlg, &hashAlg,
+                                    &mech, &mechparams);
     if (rv != SECSuccess) {
         return NULL;
     }
 
-    cx = vfy_CreateContext(key, sig, encAlg, hashAlg, hash, wincx);
-    if (sigAlgorithm->parameters.data) {
-        cx->params = SECITEM_DupItem(&sigAlgorithm->parameters);
-    }
-
-    return cx;
+    return vfy_CreateContext(key, sig, encAlg, hashAlg, mech, &mechparams,
+                             hash, PR_FALSE, wincx);
 }
 
 void
@@ -628,15 +890,17 @@ VFY_DestroyContext(VFYContext *cx, PRBool freeit)
             (*cx->hashobj->destroy)(cx->hashcx, PR_TRUE);
             cx->hashcx = NULL;
         }
+        if (cx->vfycx != NULL) {
+            (void)PK11_DestroyContext(cx->vfycx, PR_TRUE);
+            cx->vfycx = NULL;
+        }
         if (cx->key) {
             SECKEY_DestroyPublicKey(cx->key);
         }
         if (cx->pkcs1RSADigestInfo) {
             PORT_Free(cx->pkcs1RSADigestInfo);
         }
-        if (cx->params) {
-            SECITEM_FreeItem(cx->params, PR_TRUE);
-        }
+        SECITEM_FreeItem(&cx->mechparams, PR_FALSE);
         if (freeit) {
             PORT_ZFree(cx, sizeof(VFYContext));
         }
@@ -650,7 +914,17 @@ VFY_Begin(VFYContext *cx)
         (*cx->hashobj->destroy)(cx->hashcx, PR_TRUE);
         cx->hashcx = NULL;
     }
-
+    if (cx->vfycx != NULL) {
+        (void)PK11_DestroyContext(cx->vfycx, PR_TRUE);
+        cx->vfycx = NULL;
+    }
+    if (cx->mech != CKM_INVALID_MECHANISM) {
+        cx->vfycx = PK11_CreateContextByPubKey(cx->mech, CKA_VERIFY, cx->key,
+                                               &cx->mechparams, cx->wincx);
+        if (!cx->vfycx)
+            return SECFailure;
+        return SECSuccess;
+    }
     cx->hashobj = HASH_GetHashObjectByOidTag(cx->hashAlg);
     if (!cx->hashobj)
         return SECFailure; /* error code is set */
@@ -667,102 +941,87 @@ SECStatus
 VFY_Update(VFYContext *cx, const unsigned char *input, unsigned inputLen)
 {
     if (cx->hashcx == NULL) {
-        PORT_SetError(SEC_ERROR_INVALID_ARGS);
-        return SECFailure;
+        if (cx->vfycx == NULL) {
+            PORT_SetError(SEC_ERROR_INVALID_ARGS);
+            return SECFailure;
+        }
+        return PK11_DigestOp(cx->vfycx, input, inputLen);
     }
     (*cx->hashobj->update)(cx->hashcx, input, inputLen);
     return SECSuccess;
 }
 
+static SECStatus
+vfy_SingleShot(VFYContext *cx, const unsigned char *buf, int len)
+{
+    SECStatus rv;
+    /* if we have the combo mechanism, do a direct verify */
+    if (cx->mech != CKM_INVALID_MECHANISM) {
+        SECItem sig = { siBuffer, cx->u.gensig, cx->signatureLen };
+        SECItem data = { siBuffer, (unsigned char *)buf, len };
+        return PK11_VerifyWithMechanism(cx->key, cx->mech, &cx->mechparams,
+                                        &sig, &data, cx->wincx);
+    }
+    rv = VFY_Begin(cx);
+    if (rv != SECSuccess) {
+        return rv;
+    }
+    rv = VFY_Update(cx, (unsigned char *)buf, len);
+    if (rv != SECSuccess) {
+        return rv;
+    }
+    return VFY_End(cx);
+}
+
 SECStatus
 VFY_EndWithSignature(VFYContext *cx, SECItem *sig)
 {
     unsigned char final[HASH_LENGTH_MAX];
     unsigned part;
-    SECItem hash, rsasig, dsasig; /* dsasig is also used for ECDSA */
     SECStatus rv;
 
-    if ((cx->hasSignature == PR_FALSE) && (sig == NULL)) {
+    /* make sure our signature is set (either previously nor now) */
+    if (sig) {
+        rv = vfy_SetPKCS11SigFromX509Sig(cx, sig);
+        if (rv != SECSuccess) {
+            return SECFailure;
+        }
+    } else if (cx->hasSignature == PR_FALSE) {
         PORT_SetError(SEC_ERROR_INVALID_ARGS);
         return SECFailure;
     }
 
     if (cx->hashcx == NULL) {
-        PORT_SetError(SEC_ERROR_INVALID_ARGS);
-        return SECFailure;
+        unsigned int dummy;
+        if (cx->vfycx == NULL) {
+            PORT_SetError(SEC_ERROR_INVALID_ARGS);
+            return SECFailure;
+        }
+        return PK11_DigestFinal(cx->vfycx, cx->u.gensig, &dummy,
+                                cx->signatureLen);
     }
     (*cx->hashobj->end)(cx->hashcx, final, &part, sizeof(final));
+    SECItem gensig = { siBuffer, cx->u.gensig, cx->signatureLen };
+    SECItem hash = { siBuffer, final, part };
+    PORT_Assert(part <= sizeof(final));
+    /* handle the algorithm specific final call */
     switch (cx->key->keyType) {
         case ecKey:
         case dsaKey:
-            dsasig.len = checkedSignatureLen(cx->key);
-            if (dsasig.len == 0) {
-                return SECFailure;
-            }
-            if (dsasig.len > sizeof(cx->u)) {
-                PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
-                return SECFailure;
-            }
-            dsasig.data = cx->u.buffer;
-
-            if (sig) {
-                rv = decodeECorDSASignature(cx->encAlg, sig, dsasig.data,
-                                            dsasig.len);
-                if (rv != SECSuccess) {
-                    PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
-                    return SECFailure;
-                }
-            }
-            hash.data = final;
-            hash.len = part;
-            if (PK11_Verify(cx->key, &dsasig, &hash, cx->wincx) != SECSuccess) {
+            if (PK11_Verify(cx->key, &gensig, &hash, cx->wincx) != SECSuccess) {
                 PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
                 return SECFailure;
             }
             break;
         case rsaKey:
             if (cx->encAlg == SEC_OID_PKCS1_RSA_PSS_SIGNATURE) {
-                CK_RSA_PKCS_PSS_PARAMS mech;
-                SECItem mechItem = { siBuffer, (unsigned char *)&mech, sizeof(mech) };
-                PORTCheapArenaPool tmpArena;
-
-                PORT_InitCheapArena(&tmpArena, DER_DEFAULT_CHUNKSIZE);
-                rv = sec_DecodeRSAPSSParamsToMechanism(&tmpArena.arena,
-                                                       cx->params,
-                                                       &mech);
-                PORT_DestroyCheapArena(&tmpArena);
-                if (rv != SECSuccess) {
-                    return SECFailure;
-                }
-
-                rsasig.data = cx->u.buffer;
-                rsasig.len = checkedSignatureLen(cx->key);
-                if (rsasig.len == 0) {
-                    /* Error set by checkedSignatureLen */
-                    return SECFailure;
-                }
-                if (rsasig.len > sizeof(cx->u)) {
-                    PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
-                    return SECFailure;
-                }
-                if (sig) {
-                    if (sig->len != rsasig.len) {
-                        PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
-                        return SECFailure;
-                    }
-                    PORT_Memcpy(rsasig.data, sig->data, rsasig.len);
-                }
-                hash.data = final;
-                hash.len = part;
-                if (PK11_VerifyWithMechanism(cx->key, CKM_RSA_PKCS_PSS, &mechItem,
-                                             &rsasig, &hash, cx->wincx) != SECSuccess) {
+                if (PK11_VerifyWithMechanism(cx->key, CKM_RSA_PKCS_PSS,
+                                             &cx->mechparams, &gensig, &hash,
+                                             cx->wincx) != SECSuccess) {
                     PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
                     return SECFailure;
                 }
             } else {
-                SECItem digest;
-                digest.data = final;
-                digest.len = part;
                 if (sig) {
                     SECOidTag hashid;
                     PORT_Assert(cx->hashAlg != SEC_OID_UNKNOWN);
@@ -776,7 +1035,7 @@ VFY_EndWithSignature(VFYContext *cx, SECItem *sig)
                     }
                     PORT_Assert(cx->hashAlg == hashid);
                 }
-                return verifyPKCS1DigestInfo(cx, &digest);
+                return verifyPKCS1DigestInfo(cx, &hash);
             }
             break;
         default:
@@ -799,17 +1058,20 @@ VFY_End(VFYContext *cx)
 static SECStatus
 vfy_VerifyDigest(const SECItem *digest, const SECKEYPublicKey *key,
                  const SECItem *sig, SECOidTag encAlg, SECOidTag hashAlg,
-                 void *wincx)
+                 CK_MECHANISM_TYPE mech, SECItem *mechparamsp, void *wincx)
 {
     SECStatus rv;
     VFYContext *cx;
     SECItem dsasig; /* also used for ECDSA */
     rv = SECFailure;
 
-    cx = vfy_CreateContext(key, sig, encAlg, hashAlg, NULL, wincx);
+    cx = vfy_CreateContext(key, sig, encAlg, hashAlg, mech, mechparamsp,
+                           NULL, PR_TRUE, wincx);
     if (cx != NULL) {
         switch (key->keyType) {
             case rsaKey:
+                /* PSS isn't handled here for VerifyDigest. SSL
+                 * calls PK11_Verify directly */
                 rv = verifyPKCS1DigestInfo(cx, digest);
                 /* Error (if any) set by verifyPKCS1DigestInfo */
                 break;
@@ -845,7 +1107,9 @@ VFY_VerifyDigestDirect(const SECItem *digest, const SECKEYPublicKey *key,
                        const SECItem *sig, SECOidTag encAlg,
                        SECOidTag hashAlg, void *wincx)
 {
-    return vfy_VerifyDigest(digest, key, sig, encAlg, hashAlg, wincx);
+    CK_MECHANISM_TYPE mech = sec_GetCombinedMech(encAlg, hashAlg);
+    return vfy_VerifyDigest(digest, key, sig, encAlg, hashAlg, mech,
+                            NULL, wincx);
 }
 
 SECStatus
@@ -853,11 +1117,15 @@ VFY_VerifyDigest(SECItem *digest, SECKEYPublicKey *key, SECItem *sig,
                  SECOidTag algid, void *wincx)
 {
     SECOidTag encAlg, hashAlg;
-    SECStatus rv = sec_DecodeSigAlg(key, algid, NULL, &encAlg, &hashAlg);
+    CK_MECHANISM_TYPE mech;
+    SECItem mechparams;
+    SECStatus rv = sec_DecodeSigAlg(key, algid, NULL, &encAlg, &hashAlg,
+                                    &mech, &mechparams);
     if (rv != SECSuccess) {
         return SECFailure;
     }
-    return vfy_VerifyDigest(digest, key, sig, encAlg, hashAlg, wincx);
+    return vfy_VerifyDigest(digest, key, sig, encAlg, hashAlg,
+                            mech, &mechparams, wincx);
 }
 
 /*
@@ -871,42 +1139,43 @@ VFY_VerifyDigestWithAlgorithmID(const SECItem *digest,
                                 SECOidTag hashCmp, void *wincx)
 {
     SECOidTag encAlg, hashAlg;
+    CK_MECHANISM_TYPE mech;
+    SECItem mechparams;
     SECStatus rv = sec_DecodeSigAlg(key,
                                     SECOID_GetAlgorithmTag((SECAlgorithmID *)sigAlgorithm),
-                                    &sigAlgorithm->parameters, &encAlg, &hashAlg);
+                                    &sigAlgorithm->parameters, &encAlg, &hashAlg,
+                                    &mech, &mechparams);
     if (rv != SECSuccess) {
         return rv;
     }
     if (hashCmp != SEC_OID_UNKNOWN &&
         hashAlg != SEC_OID_UNKNOWN &&
         hashCmp != hashAlg) {
+        if (mechparams.data != NULL) {
+            PORT_Free(mechparams.data);
+        }
         PORT_SetError(SEC_ERROR_BAD_SIGNATURE);
         return SECFailure;
     }
-    return vfy_VerifyDigest(digest, key, sig, encAlg, hashAlg, wincx);
+    return vfy_VerifyDigest(digest, key, sig, encAlg, hashAlg,
+                            mech, &mechparams, wincx);
 }
 
 static SECStatus
 vfy_VerifyData(const unsigned char *buf, int len, const SECKEYPublicKey *key,
                const SECItem *sig, SECOidTag encAlg, SECOidTag hashAlg,
-               const SECItem *params, SECOidTag *hash, void *wincx)
+               CK_MECHANISM_TYPE mech, SECItem *mechparamsp,
+               SECOidTag *hash, void *wincx)
 {
     SECStatus rv;
     VFYContext *cx;
 
-    cx = vfy_CreateContext(key, sig, encAlg, hashAlg, hash, wincx);
+    cx = vfy_CreateContext(key, sig, encAlg, hashAlg, mech, mechparamsp,
+                           hash, PR_FALSE, wincx);
     if (cx == NULL)
         return SECFailure;
-    if (params) {
-        cx->params = SECITEM_DupItem(params);
-    }
 
-    rv = VFY_Begin(cx);
-    if (rv == SECSuccess) {
-        rv = VFY_Update(cx, (unsigned char *)buf, len);
-        if (rv == SECSuccess)
-            rv = VFY_End(cx);
-    }
+    rv = vfy_SingleShot(cx, buf, len);
 
     VFY_DestroyContext(cx, PR_TRUE);
     return rv;
@@ -918,7 +1187,9 @@ VFY_VerifyDataDirect(const unsigned char *buf, int len,
                      SECOidTag encAlg, SECOidTag hashAlg,
                      SECOidTag *hash, void *wincx)
 {
-    return vfy_VerifyData(buf, len, key, sig, encAlg, hashAlg, NULL, hash, wincx);
+    CK_MECHANISM_TYPE mech = sec_GetCombinedMech(encAlg, hashAlg);
+    return vfy_VerifyData(buf, len, key, sig, encAlg, hashAlg, mech, NULL,
+                          hash, wincx);
 }
 
 SECStatus
@@ -926,11 +1197,15 @@ VFY_VerifyData(const unsigned char *buf, int len, const SECKEYPublicKey *key,
                const SECItem *sig, SECOidTag algid, void *wincx)
 {
     SECOidTag encAlg, hashAlg;
-    SECStatus rv = sec_DecodeSigAlg(key, algid, NULL, &encAlg, &hashAlg);
+    CK_MECHANISM_TYPE mech;
+    SECItem mechparams;
+    SECStatus rv = sec_DecodeSigAlg(key, algid, NULL, &encAlg, &hashAlg,
+                                    &mech, &mechparams);
     if (rv != SECSuccess) {
         return rv;
     }
-    return vfy_VerifyData(buf, len, key, sig, encAlg, hashAlg, NULL, NULL, wincx);
+    return vfy_VerifyData(buf, len, key, sig, encAlg, hashAlg,
+                          mech, &mechparams, NULL, wincx);
 }
 
 SECStatus
@@ -942,11 +1217,14 @@ VFY_VerifyDataWithAlgorithmID(const unsigned char *buf, int len,
 {
     SECOidTag encAlg, hashAlg;
     SECOidTag sigAlg = SECOID_GetAlgorithmTag((SECAlgorithmID *)sigAlgorithm);
+    CK_MECHANISM_TYPE mech;
+    SECItem mechparams;
     SECStatus rv = sec_DecodeSigAlg(key, sigAlg,
-                                    &sigAlgorithm->parameters, &encAlg, &hashAlg);
+                                    &sigAlgorithm->parameters, &encAlg, &hashAlg,
+                                    &mech, &mechparams);
     if (rv != SECSuccess) {
         return rv;
     }
-    return vfy_VerifyData(buf, len, key, sig, encAlg, hashAlg,
-                          &sigAlgorithm->parameters, hash, wincx);
+    return vfy_VerifyData(buf, len, key, sig, encAlg, hashAlg, mech,
+                          &mechparams, hash, wincx);
 }
